@@ -75,7 +75,7 @@ nsGIFDecoder2::nsGIFDecoder2(RasterImage &aImage)
   , mCurrentRow(-1)
   , mLastFlushedRow(-1)
   , mOldColor(0)
-  , mCurrentFrame(-1)
+  , mCurrentFrameIndex(-1)
   , mCurrentPass(0)
   , mLastFlushedPass(0)
   , mGIFOpen(false)
@@ -105,7 +105,7 @@ nsGIFDecoder2::FinishInternal()
 
   // If the GIF got cut off, handle it anyway
   if (!IsSizeDecode() && mGIFOpen) {
-    if (mCurrentFrame == mGIFStruct.images_decoded)
+    if (mCurrentFrameIndex == mGIFStruct.images_decoded)
       EndImageFrame();
     PostDecodeDone(mGIFStruct.loop_count - 1);
     mGIFOpen = false;
@@ -154,20 +154,18 @@ void nsGIFDecoder2::BeginGIF()
   mGIFOpen = true;
 
   PostSize(mGIFStruct.screen_width, mGIFStruct.screen_height);
-
-  // If we're doing a size decode, we have what we came for
-  if (IsSizeDecode())
-    return;
 }
 
 //******************************************************************************
 void nsGIFDecoder2::BeginImageFrame(uint16_t aDepth)
 {
-  gfxASurface::gfxImageFormat format;
+  gfx::SurfaceFormat format;
   if (mGIFStruct.is_transparent)
-    format = gfxASurface::ImageFormatARGB32;
+    format = gfx::SurfaceFormat::B8G8R8A8;
   else
-    format = gfxASurface::ImageFormatRGB24;
+    format = gfx::SurfaceFormat::B8G8R8X8;
+
+  MOZ_ASSERT(HasSize());
 
   // Use correct format, RGB for first frame, PAL for following frames
   // and include transparency to allow for optimization of opaque images
@@ -176,21 +174,33 @@ void nsGIFDecoder2::BeginImageFrame(uint16_t aDepth)
     NeedNewFrame(mGIFStruct.images_decoded, mGIFStruct.x_offset,
                  mGIFStruct.y_offset, mGIFStruct.width, mGIFStruct.height,
                  format, aDepth);
-  } else {
+  }
+
+  // Our first full frame is automatically created by the image decoding
+  // infrastructure. Just use it as long as it matches up.
+  else if (!GetCurrentFrame()->GetRect().IsEqualEdges(nsIntRect(mGIFStruct.x_offset,
+                                                                mGIFStruct.y_offset,
+                                                                mGIFStruct.width,
+                                                                mGIFStruct.height))) {
     // Regardless of depth of input, image is decoded into 24bit RGB
     NeedNewFrame(mGIFStruct.images_decoded, mGIFStruct.x_offset,
                  mGIFStruct.y_offset, mGIFStruct.width, mGIFStruct.height,
                  format);
+  } else {
+    // Our preallocated frame matches up, with the possible exception of alpha.
+    if (format == gfx::SurfaceFormat::B8G8R8X8) {
+      GetCurrentFrame()->SetHasNoAlpha();
+    }
   }
 
-  mCurrentFrame = mGIFStruct.images_decoded;
+  mCurrentFrameIndex = mGIFStruct.images_decoded;
 }
 
 
 //******************************************************************************
 void nsGIFDecoder2::EndImageFrame()
 {
-  RasterImage::FrameAlpha alpha = RasterImage::kFrameHasAlpha;
+  FrameBlender::FrameAlpha alpha = FrameBlender::kFrameHasAlpha;
 
   // First flush all pending image data 
   if (!mGIFStruct.images_decoded) {
@@ -209,7 +219,7 @@ void nsGIFDecoder2::EndImageFrame()
     }
     // This transparency check is only valid for first frame
     if (mGIFStruct.is_transparent && !mSawTransparency) {
-      alpha = RasterImage::kFrameOpaque;
+      alpha = FrameBlender::kFrameOpaque;
     }
   }
   mCurrentRow = mLastFlushedRow = -1;
@@ -232,7 +242,7 @@ void nsGIFDecoder2::EndImageFrame()
 
   // Tell the superclass we finished a frame
   PostFrameStop(alpha,
-                RasterImage::FrameDisposalMethod(mGIFStruct.disposal_method),
+                FrameBlender::FrameDisposalMethod(mGIFStruct.disposal_method),
                 mGIFStruct.delay_time);
 
   // Reset the transparent pixel
@@ -241,7 +251,7 @@ void nsGIFDecoder2::EndImageFrame()
     mOldColor = 0;
   }
 
-  mCurrentFrame = -1;
+  mCurrentFrameIndex = -1;
 }
 
 
@@ -534,7 +544,7 @@ static void ConvertColormap(uint32_t *aColormap, uint32_t aColors)
 }
 
 void
-nsGIFDecoder2::WriteInternal(const char *aBuffer, uint32_t aCount)
+nsGIFDecoder2::WriteInternal(const char *aBuffer, uint32_t aCount, DecodeStrategy)
 {
   NS_ABORT_IF_FALSE(!HasError(), "Shouldn't call WriteInternal after error!");
 
@@ -655,6 +665,12 @@ nsGIFDecoder2::WriteInternal(const char *aBuffer, uint32_t aCount)
       mGIFStruct.screen_height = GETINT16(q + 2);
       mGIFStruct.global_colormap_depth = (q[4]&0x07) + 1;
 
+      if (IsSizeDecode()) {
+        MOZ_ASSERT(!mGIFOpen, "Gif should not be open at this point");
+        PostSize(mGIFStruct.screen_width, mGIFStruct.screen_height);
+        return;
+      }
+
       // screen_bgcolor is not used
       //mGIFStruct.screen_bgcolor = q[5];
       // q[6] = Pixel Aspect Ratio
@@ -899,19 +915,23 @@ nsGIFDecoder2::WriteInternal(const char *aBuffer, uint32_t aCount)
       mColorMask = 0xFF >> (8 - realDepth);
       BeginImageFrame(realDepth);
 
-      // We now need a new frame from the decoder framework. We leave all our
-      // data in the buffer as if it wasn't consumed, copy to our hold and return
-      // to the decoder framework.
-      uint32_t size = len + mGIFStruct.bytes_to_consume + mGIFStruct.bytes_in_hold;
-      if (size) {
-        if (SetHold(q, mGIFStruct.bytes_to_consume + mGIFStruct.bytes_in_hold, buf, len)) {
-          // Back into the decoder infrastructure so we can get called again.
-          GETN(9, gif_image_header_continue);
-          return;
+      if (NeedsNewFrame()) {
+        // We now need a new frame from the decoder framework. We leave all our
+        // data in the buffer as if it wasn't consumed, copy to our hold and return
+        // to the decoder framework.
+        uint32_t size = len + mGIFStruct.bytes_to_consume + mGIFStruct.bytes_in_hold;
+        if (size) {
+          if (SetHold(q, mGIFStruct.bytes_to_consume + mGIFStruct.bytes_in_hold, buf, len)) {
+            // Back into the decoder infrastructure so we can get called again.
+            GETN(9, gif_image_header_continue);
+            return;
+          }
         }
+        break;
+      } else {
+        // FALL THROUGH
       }
     }
-    break;
 
     case gif_image_header_continue:
     {
@@ -1037,8 +1057,8 @@ nsGIFDecoder2::WriteInternal(const char *aBuffer, uint32_t aCount)
       break;
 
     case gif_done:
-      PostDecodeDone(mGIFStruct.loop_count - 1);
-      mGIFOpen = false;
+      MOZ_ASSERT(!IsSizeDecode(), "Size decodes shouldn't reach gif_done");
+      FinishInternal();
       goto done;
 
     case gif_error:
