@@ -20,7 +20,6 @@
 #include "nsIDocShell.h"
 #include "jsapi.h"
 #include "nsCOMPtr.h"
-#include "nsCRT.h"
 #include "nsEnumeratorUtils.h"
 #include "nsEscape.h"
 #include "nsIRDFService.h"
@@ -55,7 +54,6 @@
 
 using namespace mozilla;
 
-static const int FORMAT_HTML = 2;
 static const int FORMAT_XUL = 3;
 
 //----------------------------------------------------------------------
@@ -84,7 +82,7 @@ NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsHTTPIndex)
     NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIHTTPIndex)
 NS_INTERFACE_MAP_END
 
-NS_IMPL_CYCLE_COLLECTION_1(nsHTTPIndex, mInner)
+NS_IMPL_CYCLE_COLLECTION(nsHTTPIndex, mInner)
 NS_IMPL_CYCLE_COLLECTING_ADDREF(nsHTTPIndex)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(nsHTTPIndex)
 
@@ -169,8 +167,7 @@ nsHTTPIndex::OnFTPControlLog(bool server, const char *msg)
     JSString* jsMsgStr = JS_NewUCStringCopyZ(cx, unicodeMsg.get());
     NS_ENSURE_TRUE(jsMsgStr, NS_ERROR_OUT_OF_MEMORY);
 
-    JS::AutoValueVector params(cx);
-    MOZ_ALWAYS_TRUE(params.resize(2));
+    JS::AutoValueArray<2> params(cx);
     params[0].setBoolean(server);
     params[1].setString(jsMsgStr);
 
@@ -178,9 +175,8 @@ nsHTTPIndex::OnFTPControlLog(bool server, const char *msg)
     JS_CallFunctionName(cx,
                         global,
                         "OnFTPControlLog",
-                        2,
-                        params.begin(),
-                        val.address());
+                        params,
+                        &val);
     return NS_OK;
 }
 
@@ -1257,7 +1253,7 @@ nsDirectoryViewerFactory::~nsDirectoryViewerFactory()
 }
 
 
-NS_IMPL_ISUPPORTS1(nsDirectoryViewerFactory, nsIDocumentLoaderFactory)
+NS_IMPL_ISUPPORTS(nsDirectoryViewerFactory, nsIDocumentLoaderFactory)
 
 
 
@@ -1273,7 +1269,7 @@ nsDirectoryViewerFactory::CreateInstance(const char *aCommand,
 {
   nsresult rv;
 
-  bool viewSource = (PL_strstr(aContentType,"view-source") != 0);
+  bool viewSource = aContentType && strstr(aContentType, "view-source");
 
   if (!viewSource &&
       Preferences::GetInt("network.dir.format", FORMAT_XUL) == FORMAT_XUL) {
